#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/sleep.h>

#include "includes/config.h"
#include "includes/debounce.h"
#include "includes/light_ws2812.h"

struct cRGB LED_OUTPUT[1];
struct cRGB TEMP[1];

#define MODES 13
#define BRIGHTNESSES 6
#define PWM_MAX 255

//float brightness_red = 0.3, brightness_green=0.1, brightness_blue=0.5;


ISR( PCINT0_vect){}

void try_sleep(){
	
  if( get_key_busy( 1<<KEY1 )||get_key_busy(1<<KEY0) )return;					// still busy
  set_sleep_mode( SLEEP_MODE_PWR_DOWN);		// prepare power down
  GIMSK |= (1<<PCIE);						// awake interrupt on
  sleep_mode();
  GIMSK &= ~(1<<PCIE);						// awake interrupt off
}

int main(void){
	uint8_t mode_index = 1;
	uint8_t brightness_index = BRIGHTNESSES;
	uint8_t flag = 1;	
	
	/**** Set Timer & Buttons for Debounce Functionality ****/
	KEY_DDR &= ~ALL_KEYS;
	KEY_PORT |= ALL_KEYS;

	TCCR0B = (1<<CS02)|(1<<CS00);         // divide by 1024
	TCNT0 = (uint8_t)(int16_t)-(F_CPU / 1024 * 10e-3 + 0.5);  // preload for 10ms
	TIMSK |= 1<<TOIE0;                   // enable timer interrupt	
	
	/**** Configure external Interrupts ****/
	PCMSK |= (1<<PCINT1)|(1<<PCINT2); //set Pin Change Interrupt functionality for 2 Pins
	
	
	/**** Power Reduction ****/
	PRR = (1<<PRUSI) | (1<<PRADC) | (1<<PRTIM1);
	
	sei();		// Enable global interrupts	
	
	initColors(); //can be deleted if direct initialization works
	
	TEMP[0]= colorStatic(0);
	ws2812_setleds(TEMP,1);	
	
	/************************************************************************/
	/* MAIN Action ;)                                                                     */
	/************************************************************************/
	while(1){
		
		if( get_key_short(1<<KEY0) ){
			if (mode_index == MODES)mode_index = 1;
			else mode_index += 1;
		}
		
		if (get_key_long(1<<KEY0))mode_index = 1;
		
		if( get_key_short(1<<KEY1) ){
			if (brightness_index == 1)brightness_index = BRIGHTNESSES;
			else brightness_index -= 1;			
		}
		
		if (get_key_long(1<<KEY1)) flag ^= 1;
				
		switch (mode_index){
			case 0: LED_OUTPUT[0] = colorFade();break;
			case 1: if( flag == 1){
						LED_OUTPUT[0] = stroboscope();
						 _delay_ms(10);
					}
					else LED_OUTPUT[0] = colorStatic(13);
					break;						 
			case 2: LED_OUTPUT[0] = colorStatic(0); ws2812_setleds(LED_OUTPUT,1);try_sleep(); break;
			default: break;
		}
				
		if(mode_index >= 3){			
			if(flag == 1){
				LED_OUTPUT[0] = colorPulse(mode_index - 2);
				_delay_ms(20);
			}		
			else LED_OUTPUT[0] = colorStatic(mode_index-2);// Cycle colors starting from color "1". C can be switched off here and woken up with interrupt
			
		}
		LED_OUTPUT[0].r  /= brightness_index;
		LED_OUTPUT[0].g  /= brightness_index;
		LED_OUTPUT[0].b  /= brightness_index;
		ws2812_setleds(LED_OUTPUT,1);
		
		if(flag == 0) try_sleep();
	}		
}

		
		
		/* if( get_key_press(1<<KEY0) || get_key_press(1<<KEY1)){
							white(led_index);
				_delay_ms(500);
				black(led_index);
				_delay_ms(500);
			LED_WHEEL_16[0].r = Colors(index).r;
			LED_WHEEL_16[0].g = Colors(index).g;
			LED_WHEEL_16[0].b = Colors(index).b;
			ws2812_setleds(LED_WHEEL_16,LEDS);
			_delay_ms(1000);*/

	

		/*LED_WHEEL_16[led_index].r=(uint8_t)(++value_red*brightness_red);
		LED_WHEEL_16[led_index].g=(uint8_t)(++value_green*brightness_green);
		LED_WHEEL_16[led_index].b=(uint8_t)(++value_blue*brightness_blue);
		ws2812_setleds(LED_WHEEL_16,LEDS);
		_delay_ms(5);*/
		/*if( !(PINB & (1<<PINB2)) ){
				white(led_index);
				_delay_ms(500);
				black(led_index);
				_delay_ms(500);
		}*/
		
	
/*	//while(counter--){_delay_ms(200);}
	LED_PORT |= (1<<LED_Index);  
	while(counter--){_delay_ms(200);}
	counter =10;
	LED_PORT &=~ (1<<LED_Index);
	while(counter--){_delay_ms(200);}
*/		
	/*while(1){
		
		LED_PORT |= 0xFF;//(1<<LED2);
		_delay_ms(1000);
	
		LED_PORT &= 0x00;//~(1<<LED2);
		_delay_ms(1000);
		//LED_EVENT(LED2,1000);
		LED_EVENT3(2,3);
	}*/
	//LED_EVENT3(LED2,3);
	
			/*
		LED_WHEEL_16[led_index].r=0;LED_WHEEL_16[led_index].g=value_max;LED_WHEEL_16[led_index].b=0;			// green
		ws2812_setleds(LED_WHEEL_16,16);
		_delay_ms(delay);
		
		
		LED_WHEEL_16[led_index].r=value_max;LED_WHEEL_16[led_index].g=00;LED_WHEEL_16[led_index].b=0;    // Write red to array
		ws2812_setleds(LED_WHEEL_16,16);
		_delay_ms(delay);                         // wait for 500ms.
		*/
			
		/*GICR |= (1<<INT2);
		if(led_index== 14){
			set_sleep_mode(SLEEP_MODE_PWR_DOWN);
			sleep_mode();			
		}
		GICR &= ~(1 << INT2);*/
		
		
